@@ -144,6 +144,10 @@ static int connection_close(server *srv, connection *con) {
 		srv->cur_fds--;
 	}
 
+	if (srv->srvconf.log_state_handling) {
+		log_error_write(srv, __FILE__, __LINE__, "sd",
+				"connection closed for fd", con->fd);
+	}
 	con->fd = -1;
 	connection_del(srv, con);
 	connection_set_state(srv, con, CON_STATE_CONNECT);
@@ -151,27 +155,33 @@ static int connection_close(server *srv, connection *con) {
 	return 0;
 }
 
-static void connection_handle_close_state(server *srv, connection *con) {
+static void connection_read_for_eos(server *srv, connection *con) {
 	/* we have to do the linger_on_close stuff regardless
 	 * of con->keep_alive; even non-keepalive sockets may
 	 * still have unread data, and closing before reading
 	 * it will make the client not see all our output.
 	 */
-	int len;
-	char buf[1024];
+	ssize_t len;
+	char buf[4096];
 
-	len = read(con->fd, buf, sizeof(buf));
-	if (len == 0 || (len < 0 && errno != EAGAIN && errno != EINTR) ) {
+	do {
+		len = read(con->fd, buf, sizeof(buf));
+	} while (len > 0 || (len < 0 && errno == EINTR));
+
+	if (len < 0 && errno == EAGAIN) return;
+      #if defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN
+	if (len < 0 && errno == EWOULDBLOCK) return;
+      #endif
+
+	/* 0 == len || (len < 0 && (errno is a non-recoverable error)) */
 		con->close_timeout_ts = srv->cur_ts - (HTTP_LINGER_TIMEOUT+1);
-	}
+}
+
+static void connection_handle_close_state(server *srv, connection *con) {
+	connection_read_for_eos(srv, con);
 
 	if (srv->cur_ts - con->close_timeout_ts > HTTP_LINGER_TIMEOUT) {
 		connection_close(srv, con);
-
-		if (srv->srvconf.log_state_handling) {
-			log_error_write(srv, __FILE__, __LINE__, "sd",
-					"connection closed for fd", con->fd);
-		}
 	}
 }
 
@@ -255,6 +265,16 @@ static void connection_handle_shutdown(server *srv, connection *con) {
 	srv->con_closed++;
 	connection_reset(srv, con);
 
+	/* plugins should have cleaned themselves up */
+	for (size_t i = 0; i < srv->plugins.used; ++i) {
+		plugin *p = ((plugin **)(srv->plugins.ptr))[i];
+		plugin_data *pd = p->data;
+		if (!pd || NULL == con->plugin_ctx[pd->id]) continue;
+		log_error_write(srv, __FILE__, __LINE__, "sb",
+				"missing cleanup in", p->name);
+		con->plugin_ctx[pd->id] = NULL;
+	}
+
 	/* close the connection */
 	if ((0 == shutdown(con->fd, SHUT_WR))) {
 		con->close_timeout_ts = srv->cur_ts;
@@ -278,8 +298,7 @@ static void connection_handle_response_end_state(server *srv, connection *con) {
 
 	if (con->state != CON_STATE_ERROR) srv->con_written++;
 
-	if ((con->request.content_length
-	     && (off_t)con->request.content_length > con->request_content_queue->bytes_in)
+	if (con->request.content_length != con->request_content_queue->bytes_in
 	    || con->state == CON_STATE_ERROR) {
 		/* request body is present and has not been read completely */
 		con->keep_alive = 0;
@@ -692,18 +711,10 @@ void connections_free(server *srv) {
 
 
 int connection_reset(server *srv, connection *con) {
-	size_t i;
-
 	plugins_call_connection_reset(srv, con);
 
+	connection_response_reset(srv, con);
 	con->is_readable = 1;
-	con->is_writable = 1;
-	con->http_status = 0;
-	con->file_finished = 0;
-	con->file_started = 0;
-	con->got_response = 0;
-
-	con->parsed_response = 0;
 
 	con->bytes_written = 0;
 	con->bytes_written_cur_second = 0;
@@ -717,12 +728,6 @@ int connection_reset(server *srv, connection *con) {
 	con->request.http_if_modified_since = NULL;
 	con->request.http_if_none_match = NULL;
 
-	con->response.keep_alive = 0;
-	con->response.content_length = -1;
-	con->response.transfer_encoding = 0;
-
-	con->mode = DIRECT;
-
 #define CLEAN(x) \
 	if (con->x) buffer_reset(con->x);
 
@@ -739,12 +744,6 @@ int connection_reset(server *srv, connection *con) {
 	CLEAN(uri.path_raw);
 	/* CLEAN(uri.query); */
 
-	CLEAN(physical.doc_root);
-	CLEAN(physical.path);
-	CLEAN(physical.basedir);
-	CLEAN(physical.rel_path);
-	CLEAN(physical.etag);
-
 	CLEAN(parse_request);
 
 	CLEAN(server_name);
@@ -766,28 +765,13 @@ int connection_reset(server *srv, connection *con) {
 	CLEAN(http_content_type);
 #undef CLEAN
 	con->request.content_length = 0;
+	con->request.te_chunked = 0;
 
 	array_reset(con->request.headers);
-	array_reset(con->response.headers);
 	array_reset(con->environment);
 
-	chunkqueue_reset(con->write_queue);
 	chunkqueue_reset(con->request_content_queue);
 
-	/* the plugins should cleanup themself */
-	for (i = 0; i < srv->plugins.used; i++) {
-		plugin *p = ((plugin **)(srv->plugins.ptr))[i];
-		plugin_data *pd = p->data;
-
-		if (!pd) continue;
-
-		if (con->plugin_ctx[pd->id] != NULL) {
-			log_error_write(srv, __FILE__, __LINE__, "sb", "missing cleanup in", p->name);
-		}
-
-		con->plugin_ctx[pd->id] = NULL;
-	}
-
 	/* The cond_cache gets reset in response.c */
 	/* config_cond_cache_reset(srv, con); */
 
@@ -971,13 +955,7 @@ static handler_t connection_handle_fdevent(server *srv, void *context, int reven
 
 	if (con->state == CON_STATE_CLOSE) {
 		/* flush the read buffers */
-		int len;
-		char buf[1024];
-
-		len = read(con->fd, buf, sizeof(buf));
-		if (len == 0 || (len < 0 && errno != EAGAIN && errno != EINTR) ) {
-			con->close_timeout_ts = srv->cur_ts - (HTTP_LINGER_TIMEOUT+1);
-		}
+		connection_read_for_eos(srv, con);
 	}
 
 
@@ -1027,7 +1005,11 @@ connection *connection_accept(server *srv, server_socket *srv_socket) {
 	cnt_len = sizeof(cnt_addr);
 
 #if defined(SOCK_CLOEXEC) && defined(SOCK_NONBLOCK)
+#if defined(__NetBSD__)
+	cnt = paccept(srv_socket->fd, (struct sockaddr *) &cnt_addr, &cnt_len, NULL, SOCK_CLOEXEC | SOCK_NONBLOCK);
+#else
 	cnt = accept4(srv_socket->fd, (struct sockaddr *) &cnt_addr, &cnt_len, SOCK_CLOEXEC | SOCK_NONBLOCK);
+#endif
 #else
 	cnt = accept(srv_socket->fd, (struct sockaddr *) &cnt_addr, &cnt_len);
 #endif
@@ -1221,7 +1203,7 @@ int connection_state_machine(server *srv, connection *con) {
 								plugins_call_connection_reset(srv, con);
 
 								if (con->request.content_length) {
-									if ((off_t)con->request.content_length != chunkqueue_length(con->request_content_queue)) {
+									if (con->request.content_length != con->request_content_queue->bytes_in) {
 										con->keep_alive = 0;
 									}
 									con->request.content_length = 0;
