@@ -18,16 +18,13 @@
  *   TODO: db connection pool (if asynchronous requests)
  */
 
-#include "plugin.h"
-
-#ifdef HAVE_MYSQL
-
 #include <mysql.h>
 
 #include "server.h"
 #include "http_auth.h"
 #include "log.h"
 #include "md5.h"
+#include "plugin.h"
 
 #include <ctype.h>
 #include <errno.h>
@@ -178,6 +175,7 @@ FREE_FUNC(mod_authn_mysql_free) {
         }
         free(p->config_storage);
     }
+    mod_authn_mysql_sock_close(&p->conf);
 
     free(p);
 
@@ -344,7 +342,11 @@ static int mod_authn_mysql_password_cmp(const char *userpw, unsigned long userpw
             char *crypted;
           #if defined(HAVE_CRYPT_R)
             struct crypt_data crypt_tmp_data;
+           #ifdef _AIX
+            memset(&crypt_tmp_data, 0, sizeof(crypt_tmp_data));
+           #else
             crypt_tmp_data.initialized = 0;
+           #endif
           #endif
             memcpy(salt, saltb, slen);
             salt[slen] = '\0';
@@ -542,13 +544,3 @@ int mod_authn_mysql_plugin_init(plugin *p) {
 
     return 0;
 }
-
-#else
-
-int mod_authn_mysql_plugin_init(plugin *p);
-int mod_authn_mysql_plugin_init(plugin *p) {
-    UNUSED(p);
-    return -1;
-}
-
-#endif
