@@ -71,6 +71,7 @@ typedef struct {
 	array  *compress;
 	off_t   compress_max_filesize; /** max filesize in kb */
 	int     allowed_encodings;
+	double  max_loadavg;
 } plugin_config;
 
 typedef struct {
@@ -177,6 +178,7 @@ SETDEFAULTS_FUNC(mod_compress_setdefaults) {
 		{ "compress.filetype",              NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },
 		{ "compress.max-filesize",          NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },
 		{ "compress.allowed-encodings",     NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },
+		{ "compress.max-loadavg",           NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },
 		{ NULL,                             NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 
@@ -192,11 +194,14 @@ SETDEFAULTS_FUNC(mod_compress_setdefaults) {
 		s->compress = array_init();
 		s->compress_max_filesize = 0;
 		s->allowed_encodings = 0;
+		s->max_loadavg = 0.0;
 
 		cv[0].destination = s->compress_cache_dir;
 		cv[1].destination = s->compress;
 		cv[2].destination = &(s->compress_max_filesize);
 		cv[3].destination = encodings_arr; /* temp array for allowed encodings list */
+		cv[4].destination = srv->tmp_buf;
+		buffer_string_set_length(srv->tmp_buf, 0);
 
 		p->config_storage[i] = s;
 
@@ -204,10 +209,16 @@ SETDEFAULTS_FUNC(mod_compress_setdefaults) {
 			return HANDLER_ERROR;
 		}
 
+		if (!buffer_string_is_empty(srv->tmp_buf)) {
+			s->max_loadavg = strtod(srv->tmp_buf->ptr, NULL);
+		}
+
 		if (encodings_arr->used) {
 			size_t j = 0;
 			for (j = 0; j < encodings_arr->used; j++) {
+#if defined(USE_ZLIB) || defined(USE_BZ2LIB)
 				data_string *ds = (data_string *)encodings_arr->data[j];
+#endif
 #ifdef USE_ZLIB
 				if (NULL != strstr(ds->value->ptr, "gzip"))
 					s->allowed_encodings |= HTTP_ACCEPT_ENCODING_GZIP | HTTP_ACCEPT_ENCODING_X_GZIP;
@@ -432,6 +443,20 @@ static int deflate_file_to_buffer_bzip2(server *srv, connection *con, plugin_dat
 }
 #endif
 
+static void mod_compress_note_ratio(server *srv, connection *con, off_t in, off_t out) {
+    /* store compression ratio in con->environment
+     * for possible logging by mod_accesslog
+     * (late in response handling, so not seen by most other modules) */
+    /*(should be called only at end of successful response compression)*/
+    char ratio[LI_ITOSTRING_LENGTH];
+    if (0 == in) return;
+    li_itostrn(ratio, sizeof(ratio), out * 100 / in);
+    array_set_key_value(con->environment,
+                        CONST_STR_LEN("ratio"),
+                        ratio, strlen(ratio));
+    UNUSED(srv);
+}
+
 static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, buffer *fn, stat_cache_entry *sce, int type) {
 	int ifd, ofd;
 	int ret;
@@ -440,6 +465,7 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 #endif
 	void *start;
 	const char *filename = fn->ptr;
+	stat_cache_entry *sce_ofn;
 	ssize_t r;
 
 	/* overflow */
@@ -481,6 +507,21 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 
 	buffer_append_string_buffer(p->ofn, sce->etag);
 
+	if (HANDLER_ERROR != stat_cache_get_entry(srv, con, p->ofn, &sce_ofn)) {
+		if (0 == sce->st.st_size) return -1; /* cache file being created */
+		/* cache-entry exists */
+#if 0
+		log_error_write(srv, __FILE__, __LINE__, "bs", p->ofn, "compress-cache hit");
+#endif
+		mod_compress_note_ratio(srv, con, sce->st.st_size, sce_ofn->st.st_size);
+		buffer_copy_buffer(con->physical.path, p->ofn);
+		return 0;
+	}
+
+	if (0.0 < p->conf.max_loadavg && p->conf.max_loadavg < srv->srvconf.loadavg[0]) {
+		return -1;
+	}
+
 	if (-1 == mkdir_for_file(p->ofn->ptr)) {
 		log_error_write(srv, __FILE__, __LINE__, "sb", "couldn't create directory for file", p->ofn);
 		return -1;
@@ -488,13 +529,7 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 
 	if (-1 == (ofd = open(p->ofn->ptr, O_WRONLY | O_CREAT | O_EXCL | O_BINARY, 0600))) {
 		if (errno == EEXIST) {
-			/* cache-entry exists */
-#if 0
-			log_error_write(srv, __FILE__, __LINE__, "bs", p->ofn, "compress-cache hit");
-#endif
-			buffer_copy_buffer(con->physical.path, p->ofn);
-
-			return 0;
+			return -1; /* cache file being created */
 		}
 
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "creating cachefile", p->ofn, "failed", strerror(errno));
@@ -610,6 +645,8 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 	}
 
 	buffer_copy_buffer(con->physical.path, p->ofn);
+	mod_compress_note_ratio(srv, con, sce->st.st_size,
+				(off_t)buffer_string_length(p->b));
 
 	return 0;
 }
@@ -632,6 +669,9 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 
 	if (sce->st.st_size > 128 * 1024 * 1024) return -1;
 
+	if (0.0 < p->conf.max_loadavg && p->conf.max_loadavg < srv->srvconf.loadavg[0]) {
+		return -1;
+	}
 
 	if (-1 == (ifd = open(fn->ptr, O_RDONLY | O_BINARY))) {
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "opening plain-file", fn, "failed", strerror(errno));
@@ -697,6 +737,8 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 
 	if (ret != 0) return -1;
 
+	mod_compress_note_ratio(srv, con, sce->st.st_size,
+				(off_t)buffer_string_length(p->b));
 	chunkqueue_reset(con->write_queue);
 	chunkqueue_append_buffer(con->write_queue, p->b);
 
@@ -719,6 +761,7 @@ static int mod_compress_patch_connection(server *srv, connection *con, plugin_da
 	PATCH(compress);
 	PATCH(compress_max_filesize);
 	PATCH(allowed_encodings);
+	PATCH(max_loadavg);
 
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
@@ -740,6 +783,8 @@ static int mod_compress_patch_connection(server *srv, connection *con, plugin_da
 				PATCH(compress_max_filesize);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("compress.allowed-encodings"))) {
 				PATCH(allowed_encodings);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("compress.max-loadavg"))) {
+				PATCH(max_loadavg);
 			}
 		}
 	}
