@@ -174,6 +174,7 @@ typedef struct {
 	unsigned short	work_block_size;
 	unsigned short	sync_flush;
 	short		compression_level;
+	double		max_loadavg;
 } plugin_config;
 
 typedef struct {
@@ -274,6 +275,7 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 		{ "deflate.compression-level",     NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },
 		{ "deflate.output-buffer-size",    NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },
 		{ "deflate.work-block-size",       NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },
+		{ "deflate.max-loadavg",           NULL, T_CONFIG_STRING,T_CONFIG_SCOPE_CONNECTION },
 		{ NULL,                            NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 
@@ -291,6 +293,7 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 		s->work_block_size = 2048;
 		s->sync_flush = 0;
 		s->compression_level = -1;
+		s->max_loadavg = 0.0;
 
 		array_reset(p->encodings); /* temp array for allowed encodings list */
 
@@ -301,6 +304,8 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 		cv[4].destination = &(s->compression_level);
 		cv[5].destination = &(s->output_buffer_size);
 		cv[6].destination = &(s->work_block_size);
+		cv[7].destination = p->tmp_buf;
+		buffer_string_set_length(p->tmp_buf, 0);
 
 		p->config_storage[i] = s;
 
@@ -315,6 +320,10 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 			return HANDLER_ERROR;
 		}
 
+		if (!buffer_string_is_empty(p->tmp_buf)) {
+			s->max_loadavg = strtod(p->tmp_buf->ptr, NULL);
+		}
+
 		if (p->encodings->used) {
 			size_t j = 0;
 			for (j = 0; j < p->encodings->used; j++) {
@@ -352,6 +361,17 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 					     |  HTTP_ACCEPT_ENCODING_X_BZIP2;
 #endif
 		}
+
+		/* mod_deflate matches mimetype as prefix of Content-Type
+		 * so ignore '*' at end of mimetype for end-user flexibility
+		 * in specifying trailing wildcard to grouping of mimetypes */
+		for (size_t m = 0; m < s->mimetypes->used; ++m) {
+			buffer *mimetype = ((data_string *)s->mimetypes->data[m])->value;
+			size_t len = buffer_string_length(mimetype);
+			if (len > 2 && mimetype->ptr[len-1] == '*') {
+				buffer_string_set_length(mimetype, len-1);
+			}
+		}
 	}
 
 	return HANDLER_GO_ON;
@@ -636,6 +656,20 @@ static int mod_deflate_stream_flush(server *srv, connection *con, handler_ctx *h
 	}
 }
 
+static void mod_deflate_note_ratio(server *srv, connection *con, handler_ctx *hctx) {
+    /* store compression ratio in con->environment
+     * for possible logging by mod_accesslog
+     * (late in response handling, so not seen by most other modules) */
+    /*(should be called only at end of successful response compression)*/
+    char ratio[LI_ITOSTRING_LENGTH];
+    if (0 == hctx->bytes_in) return;
+    li_itostrn(ratio, sizeof(ratio), hctx->bytes_out * 100 / hctx->bytes_in);
+    array_set_key_value(con->environment,
+                        CONST_STR_LEN("ratio"),
+                        ratio, strlen(ratio));
+    UNUSED(srv);
+}
+
 static int mod_deflate_stream_end(server *srv, handler_ctx *hctx) {
 	switch(hctx->compression_type) {
 #ifdef USE_ZLIB
@@ -668,9 +702,6 @@ static void deflate_compress_cleanup(server *srv, connection *con, handler_ctx *
 	}
       #endif
 
-	/* future: might store ((double)hctx->bytes_out / hctx->bytes_in)
-	 * in con->environment, for possible logging by mod_accesslog */
-
 	handler_ctx_free(hctx);
 }
 
@@ -907,6 +938,7 @@ static int mod_deflate_patch_connection(server *srv, connection *con, plugin_dat
 	PATCH(compression_level);
 	PATCH(output_buffer_size);
 	PATCH(work_block_size);
+	PATCH(max_loadavg);
 
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
@@ -934,6 +966,8 @@ static int mod_deflate_patch_connection(server *srv, connection *con, plugin_dat
 				PATCH(output_buffer_size);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("deflate.work-block-size"))) {
 				PATCH(work_block_size);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("deflate.max-loadavg"))) {
+				PATCH(max_loadavg);
 			}
 		}
 	}
@@ -1120,6 +1154,10 @@ CONNECTION_FUNC(mod_deflate_handle_response_start) {
 		}
 	}
 
+	if (0.0 < p->conf.max_loadavg && p->conf.max_loadavg < srv->srvconf.loadavg[0]) {
+		return HANDLER_GO_ON;
+	}
+
 	/* update ETag, if ETag response header is set */
 	if (etaglen) {
 		/* modify ETag response header in-place to remove '"' and append '-label"' */
@@ -1179,6 +1217,9 @@ CONNECTION_FUNC(mod_deflate_handle_response_start) {
 
 	rc = deflate_compress_response(srv, con, hctx);
 	if (HANDLER_GO_ON != rc) {
+		if (HANDLER_FINISHED == rc) {
+			mod_deflate_note_ratio(srv, con, hctx);
+		}
 		deflate_compress_cleanup(srv, con, hctx);
 		if (HANDLER_ERROR == rc) return HANDLER_ERROR;
 	}
