@@ -8,7 +8,6 @@
 
 #include "response.h"
 #include "stat_cache.h"
-#include "stream.h"
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -52,16 +51,16 @@ typedef struct {
 typedef struct {
 	unsigned short dir_listing;
 	unsigned short hide_dot_files;
-	unsigned short show_readme;
 	unsigned short hide_readme_file;
 	unsigned short encode_readme;
-	unsigned short show_header;
 	unsigned short hide_header_file;
 	unsigned short encode_header;
 	unsigned short auto_layout;
 
 	excludes_buffer *excludes;
 
+	buffer *show_readme;
+	buffer *show_header;
 	buffer *external_css;
 	buffer *external_js;
 	buffer *encoding;
@@ -178,6 +177,8 @@ FREE_FUNC(mod_dirlisting_free) {
 			if (!s) continue;
 
 			excludes_buffer_free(s->excludes);
+			buffer_free(s->show_readme);
+			buffer_free(s->show_header);
 			buffer_free(s->external_css);
 			buffer_free(s->external_js);
 			buffer_free(s->encoding);
@@ -225,9 +226,9 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
 		{ CONFIG_HIDE_DOTFILES,    NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 2 */
 		{ CONFIG_EXTERNAL_CSS,     NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 3 */
 		{ CONFIG_ENCODING,         NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 4 */
-		{ CONFIG_SHOW_README,      NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 5 */
+		{ CONFIG_SHOW_README,      NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 5 */
 		{ CONFIG_HIDE_README_FILE, NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 6 */
-		{ CONFIG_SHOW_HEADER,      NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 7 */
+		{ CONFIG_SHOW_HEADER,      NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 7 */
 		{ CONFIG_HIDE_HEADER_FILE, NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 8 */
 		{ CONFIG_DIR_LISTING,      NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 9 */
 		{ CONFIG_SET_FOOTER,       NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 10 */
@@ -251,12 +252,12 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
 		s = calloc(1, sizeof(plugin_config));
 		s->excludes = excludes_buffer_init();
 		s->dir_listing = 0;
+		s->show_readme = buffer_init();
+		s->show_header = buffer_init();
 		s->external_css = buffer_init();
 		s->external_js = buffer_init();
 		s->hide_dot_files = 1;
-		s->show_readme = 0;
 		s->hide_readme_file = 0;
-		s->show_header = 0;
 		s->hide_header_file = 0;
 		s->encode_readme = 1;
 		s->encode_header = 1;
@@ -270,9 +271,9 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
 		cv[2].destination = &(s->hide_dot_files);
 		cv[3].destination = s->external_css;
 		cv[4].destination = s->encoding;
-		cv[5].destination = &(s->show_readme);
+		cv[5].destination = s->show_readme;
 		cv[6].destination = &(s->hide_readme_file);
-		cv[7].destination = &(s->show_header);
+		cv[7].destination = s->show_header;
 		cv[8].destination = &(s->hide_header_file);
 		cv[9].destination = &(s->dir_listing); /* old name */
 		cv[10].destination = s->set_footer;
@@ -324,6 +325,24 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
 			}
 #endif
 		}
+
+		if (!buffer_string_is_empty(s->show_readme)) {
+			if (buffer_is_equal_string(s->show_readme, CONST_STR_LEN("enable"))) {
+				buffer_copy_string_len(s->show_readme, CONST_STR_LEN("README.txt"));
+			}
+			else if (buffer_is_equal_string(s->show_readme, CONST_STR_LEN("disable"))) {
+				buffer_string_set_length(s->show_readme, 0);
+			}
+		}
+
+		if (!buffer_string_is_empty(s->show_header)) {
+			if (buffer_is_equal_string(s->show_header, CONST_STR_LEN("enable"))) {
+				buffer_copy_string_len(s->show_header, CONST_STR_LEN("HEADER.txt"));
+			}
+			else if (buffer_is_equal_string(s->show_header, CONST_STR_LEN("disable"))) {
+				buffer_string_set_length(s->show_header, 0);
+			}
+		}
 	}
 
 	return HANDLER_GO_ON;
@@ -706,18 +725,24 @@ static void http_list_directory_header(server *srv, connection *con, plugin_data
 
 	if (p->conf.auto_layout) {
 		buffer_append_string_len(out, CONST_STR_LEN(
-			"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n"
-			"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n"
+			"<!DOCTYPE html>\n"
+			"<html>\n"
 			"<head>\n"
-			"<title>Index of "
 		));
+		if (!buffer_string_is_empty(p->conf.encoding)) {
+			buffer_append_string_len(out, CONST_STR_LEN("<meta charset=\""));
+			buffer_append_string_buffer(out, p->conf.encoding);
+			buffer_append_string_len(out, CONST_STR_LEN("\">\n"));
+		}
+		buffer_append_string_len(out, CONST_STR_LEN("<title>Index of "));
 		buffer_append_string_encoded(out, CONST_BUF_LEN(con->uri.path), ENCODING_MINIMAL_XML);
 		buffer_append_string_len(out, CONST_STR_LEN("</title>\n"));
 
 		if (!buffer_string_is_empty(p->conf.external_css)) {
+			buffer_append_string_len(out, CONST_STR_LEN("<meta name=\"viewport\" content=\"initial-scale=1\">"));
 			buffer_append_string_len(out, CONST_STR_LEN("<link rel=\"stylesheet\" type=\"text/css\" href=\""));
 			buffer_append_string_buffer(out, p->conf.external_css);
-			buffer_append_string_len(out, CONST_STR_LEN("\" />\n"));
+			buffer_append_string_len(out, CONST_STR_LEN("\">\n"));
 		} else {
 			buffer_append_string_len(out, CONST_STR_LEN(
 				"<style type=\"text/css\">\n"
@@ -757,13 +782,12 @@ static void http_list_directory_header(server *srv, connection *con, plugin_data
 		buffer_append_string_len(out, CONST_STR_LEN("</head>\n<body>\n"));
 	}
 
-	/* HEADER.txt */
-	if (p->conf.show_header) {
+	if (!buffer_string_is_empty(p->conf.show_header)) {
 		/* if we have a HEADER file, display it in <pre class="header"></pre> */
 
 		buffer_copy_buffer(p->tmp_buf, con->physical.path);
 		buffer_append_slash(p->tmp_buf);
-		buffer_append_string_len(p->tmp_buf, CONST_STR_LEN("HEADER.txt"));
+		buffer_append_string_buffer(p->tmp_buf, p->conf.show_header);
 
 		http_list_directory_include_file(out, p->tmp_buf, "header", p->conf.encode_header);
 	}
@@ -801,26 +825,18 @@ static void http_list_directory_footer(server *srv, connection *con, plugin_data
 		"</div>\n"
 	));
 
-	if (p->conf.show_readme) {
+	if (!buffer_string_is_empty(p->conf.show_readme)) {
 		/* if we have a README file, display it in <pre class="readme"></pre> */
 
 		buffer_copy_buffer(p->tmp_buf,  con->physical.path);
 		buffer_append_slash(p->tmp_buf);
-		buffer_append_string_len(p->tmp_buf, CONST_STR_LEN("README.txt"));
+		buffer_append_string_buffer(p->tmp_buf, p->conf.show_readme);
 
 		http_list_directory_include_file(out, p->tmp_buf, "readme", p->conf.encode_readme);
 	}
 
 	if(p->conf.auto_layout) {
 
-		if (!buffer_string_is_empty(p->conf.external_js)) {
-			buffer_append_string_len(out, CONST_STR_LEN("<script type=\"text/javascript\" src=\""));
-			buffer_append_string_buffer(out, p->conf.external_js);
-			buffer_append_string_len(out, CONST_STR_LEN("\" />\n"));
-		} else if (buffer_is_empty(p->conf.external_js)) {
-			http_dirlist_append_js_table_resort(out, con);
-		}
-
 		buffer_append_string_len(out, CONST_STR_LEN(
 			"<div class=\"foot\">"
 		));
@@ -833,6 +849,17 @@ static void http_list_directory_footer(server *srv, connection *con, plugin_data
 
 		buffer_append_string_len(out, CONST_STR_LEN(
 			"</div>\n"
+		));
+
+		if (!buffer_string_is_empty(p->conf.external_js)) {
+			buffer_append_string_len(out, CONST_STR_LEN("<script type=\"text/javascript\" src=\""));
+			buffer_append_string_buffer(out, p->conf.external_js);
+			buffer_append_string_len(out, CONST_STR_LEN("\"></script>\n"));
+		} else if (buffer_is_empty(p->conf.external_js)) {
+			http_dirlist_append_js_table_resort(out, con);
+		}
+
+		buffer_append_string_len(out, CONST_STR_LEN(
 			"</body>\n"
 			"</html>\n"
 		));
@@ -915,12 +942,12 @@ static int http_list_directory(server *srv, connection *con, plugin_data *p, buf
 				continue;
 		}
 
-		if (p->conf.hide_readme_file) {
-			if (strcmp(dent->d_name, "README.txt") == 0)
+		if (p->conf.hide_readme_file && !buffer_string_is_empty(p->conf.show_readme)) {
+			if (strcmp(dent->d_name, p->conf.show_readme->ptr) == 0)
 				continue;
 		}
-		if (p->conf.hide_header_file) {
-			if (strcmp(dent->d_name, "HEADER.txt") == 0)
+		if (p->conf.hide_header_file && !buffer_string_is_empty(p->conf.show_header)) {
+			if (strcmp(dent->d_name, p->conf.show_header->ptr) == 0)
 				continue;
 		}
 
@@ -994,13 +1021,6 @@ static int http_list_directory(server *srv, connection *con, plugin_data *p, buf
 	if (files.used) http_dirls_sort(files.ent, files.used);
 
 	out = buffer_init();
-	buffer_copy_string_len(out, CONST_STR_LEN("<?xml version=\"1.0\" encoding=\""));
-	if (buffer_string_is_empty(p->conf.encoding)) {
-		buffer_append_string_len(out, CONST_STR_LEN("iso-8859-1"));
-	} else {
-		buffer_append_string_buffer(out, p->conf.encoding);
-	}
-	buffer_append_string_len(out, CONST_STR_LEN("\"?>\n"));
 	http_list_directory_header(srv, con, p, out);
 
 	/* directories */
