@@ -353,8 +353,9 @@ static handler_t process_rewrite_rules(server *srv, connection *con, plugin_data
 		hctx = con->plugin_ctx[p->id];
 
 		if (hctx->loops++ > 100) {
-			log_error_write(srv, __FILE__, __LINE__,  "s",
-					"ENDLESS LOOP IN rewrite-rule DETECTED ... aborting request, perhaps you want to use url.rewrite-once instead of url.rewrite-repeat");
+			data_config *dc = p->conf.context;
+			log_error_write(srv, __FILE__, __LINE__,  "SbbSBS",
+					"ENDLESS LOOP IN rewrite-rule DETECTED ... aborting request, perhaps you want to use url.rewrite-once instead of url.rewrite-repeat ($", dc->comp_key, dc->op, "\"", dc->string, "\")");
 
 			return HANDLER_ERROR;
 		}
