@@ -43,7 +43,7 @@ int ssi_val_tobool(ssi_val_t *B) {
 	}
 }
 
-static int ssi_expr_tokenizer(server *srv, connection *con, plugin_data *p,
+static int ssi_expr_tokenizer(server *srv, connection *con, handler_ctx *p,
 			      ssi_tokenizer_t *t, int *token_id, buffer *token) {
 	int tid = 0;
 	size_t i;
@@ -272,7 +272,7 @@ static int ssi_expr_tokenizer(server *srv, connection *con, plugin_data *p,
 	return 0;
 }
 
-int ssi_eval_expr(server *srv, connection *con, plugin_data *p, const char *expr) {
+int ssi_eval_expr(server *srv, connection *con, handler_ctx *p, const char *expr) {
 	ssi_tokenizer_t t;
 	void *pParser;
 	int token_id;
