@@ -994,7 +994,7 @@ int network_init(server *srv) {
 					s->ssl_pemfile);
 			return -1;
 		}
-		SSL_CTX_set_default_read_ahead(s->ssl_ctx, 1);
+		SSL_CTX_set_default_read_ahead(s->ssl_ctx, s->ssl_read_ahead);
 		SSL_CTX_set_mode(s->ssl_ctx,  SSL_CTX_get_mode(s->ssl_ctx)
 					    | SSL_MODE_ENABLE_PARTIAL_WRITE
 					    | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER
