@@ -181,12 +181,13 @@ static void https_add_ssl_client_entries(server *srv, connection *con) {
 
 	{
 		ASN1_INTEGER *xsn = X509_get_serialNumber(xs);
-		BIGNUM serialBN;
-		char *serialHex = BN_bn2hex(ASN1_INTEGER_to_BN(xsn, &serialBN));
+		BIGNUM *serialBN = ASN1_INTEGER_to_BN(xsn, NULL);
+		char *serialHex = BN_bn2hex(serialBN);
 		array_set_key_value(con->environment,
 				    CONST_STR_LEN("SSL_CLIENT_M_SERIAL"),
 				    serialHex, strlen(serialHex));
 		OPENSSL_free(serialHex);
+		BN_free(serialBN);
 	}
 
 	if (!buffer_string_is_empty(con->conf.ssl_verifyclient_username)) {
@@ -220,37 +221,6 @@ static void https_add_ssl_client_entries(server *srv, connection *con) {
 	}
 	X509_free(xs);
 }
-
-void http_cgi_ssl_env(server *srv, connection *con) {
-    const char *s;
-    const SSL_CIPHER *cipher;
-    UNUSED(srv);
-
-    if (!con->ssl) return;
-
-    s = SSL_get_version(con->ssl);
-    array_set_key_value(con->environment,
-                        CONST_STR_LEN("SSL_PROTOCOL"),
-                        s, strlen(s));
-
-    if ((cipher = SSL_get_current_cipher(con->ssl))) {
-        int usekeysize, algkeysize;
-        char buf[LI_ITOSTRING_LENGTH];
-        s = SSL_CIPHER_get_name(cipher);
-        array_set_key_value(con->environment,
-                            CONST_STR_LEN("SSL_CIPHER"),
-                            s, strlen(s));
-        usekeysize = SSL_CIPHER_get_bits(cipher, &algkeysize);
-        li_itostrn(buf, sizeof(buf), usekeysize);
-        array_set_key_value(con->environment,
-                            CONST_STR_LEN("SSL_CIPHER_USEKEYSIZE"),
-                            buf, strlen(buf));
-        li_itostrn(buf, sizeof(buf), algkeysize);
-        array_set_key_value(con->environment,
-                            CONST_STR_LEN("SSL_CIPHER_ALGKEYSIZE"),
-                            buf, strlen(buf));
-    }
-}
 #endif
 
 
