@@ -1,6 +1,5 @@
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include "first.h"
+
 #include "settings.h"
 
 #include "safe_memclear.h"
@@ -41,7 +40,7 @@ static void* safe_memset(void *s, int c, size_t n)
 
 void safe_memclear(void *s, size_t n) {
 #if defined(HAVE_MEMSET_S)
-	memset_s(s, 0, n);
+	memset_s(s, n, 0, n);
 #elif defined(HAVE_EXPLICIT_BZERO)
 	explicit_bzero(s, n);
 #else
