@@ -244,7 +244,6 @@ static server *server_init(void) {
 	}
 
 	li_rand_reseed();
-	li_rand_bytes((unsigned char *)srv->entropy, (int)sizeof(srv->entropy));
 
 	srv->cur_ts = time(NULL);
 	srv->startup_ts = srv->cur_ts;
@@ -269,6 +268,9 @@ static server *server_init(void) {
 	srv->srvconf.http_host_normalize = 0;
 	srv->srvconf.high_precision_timestamps = 0;
 	srv->srvconf.max_request_field_size = 8192;
+	srv->srvconf.loadavg[0] = 0.0;
+	srv->srvconf.loadavg[1] = 0.0;
+	srv->srvconf.loadavg[2] = 0.0;
 
 	/* use syslog */
 	srv->errorlog_fd = STDERR_FILENO;
@@ -377,18 +379,20 @@ static void server_free(server *srv) {
 
 #ifdef USE_OPENSSL
 	if (srv->ssl_is_init) {
-		CRYPTO_cleanup_all_ex_data();
-		ERR_free_strings();
 	      #if   OPENSSL_VERSION_NUMBER >= 0x10100000L \
 		&& !defined(LIBRESSL_VERSION_NUMBER)
 		/*(OpenSSL libraries handle thread init and deinit)
 		 * https://github.com/openssl/openssl/pull/1048 */
-	      #elif OPENSSL_VERSION_NUMBER >= 0x10000000L
-		ERR_remove_thread_state(NULL);
 	      #else
+		CRYPTO_cleanup_all_ex_data();
+		ERR_free_strings();
+	       #if OPENSSL_VERSION_NUMBER >= 0x10000000L
+		ERR_remove_thread_state(NULL);
+	       #else
 		ERR_remove_state(0);
-	      #endif
+	       #endif
 		EVP_cleanup();
+	      #endif
 	}
 #endif
 	li_rand_cleanup();
@@ -558,11 +562,6 @@ static void show_features (void) {
 #else
       "\t- poll (Unix)\n"
 #endif
-#ifdef USE_LINUX_SIGIO
-      "\t+ rt-signals (Linux 2.4+)\n"
-#else
-      "\t- rt-signals (Linux 2.4+)\n"
-#endif
 #ifdef USE_LINUX_EPOLL
       "\t+ epoll (Linux 2.6)\n"
 #else
@@ -1583,6 +1582,15 @@ int main (int argc, char **argv) {
 					graceful_shutdown = 2; /* value 2 indicates idle timeout */
 				}
 
+			      #ifdef HAVE_GETLOADAVG
+				/* refresh loadavg data every 30 seconds */
+				if (srv->srvconf.loadts + 30 < min_ts) {
+					if (-1 != getloadavg(srv->srvconf.loadavg, 3)) {
+						srv->srvconf.loadts = min_ts;
+					}
+				}
+			      #endif
+
 				/* cleanup stat-cache */
 				stat_cache_trigger_cleanup(srv);
 				/**
