@@ -1,13 +1,6 @@
 #ifndef _LIGHTTPD_SETTINGS_H_
 #define _LIGHTTPD_SETTINGS_H_
-
-#ifndef _GNU_SOURCE
-# define _GNU_SOURCE
-#endif
-
-#ifndef __USE_GNU
-# define __USE_GNU /* a hack in my eyes, <fcntl.h> F_SETSIG should work with _GNU_SOURCE */
-#endif
+#include "first.h"
 
 #ifdef __GNUC__
 # define LI_NORETURN __attribute__((noreturn))
@@ -15,6 +8,8 @@
 # define LI_NORETURN
 #endif
 
+#define UNUSED(x) ( (void)(x) )
+
 #define BV(x) (1 << x)
 
 #define INET_NTOP_CACHE_MAX 4
@@ -51,13 +46,4 @@ typedef enum { HANDLER_UNSET,
 
 #define HTTP_LINGER_TIMEOUT 5
 
-/* we use it in a enum */
-#ifdef TRUE
-#undef TRUE
-#endif
-
-#ifdef FALSE
-#undef FALSE
-#endif
-
 #endif
